@@ -9,8 +9,8 @@ STUMPY_MEAN_STD_NUM_CHUNKS = 1
 STUMPY_MEAN_STD_MAX_ITER = 10
 STUMPY_DENOM_THRESHOLD = 1e-14
 STUMPY_STDDEV_THRESHOLD = 1e-7
-STUMPY_D_SQUARED_THRESHOLD = 1e-14
+STUMPY_P_NORM_THRESHOLD = 1e-14
 STUMPY_TEST_PRECISION = 5
-STUMPY_MAX_SQUARED_DISTANCE = np.finfo(np.float64).max
-STUMPY_MAX_DISTANCE = np.sqrt(STUMPY_MAX_SQUARED_DISTANCE)
+STUMPY_MAX_P_NORM_DISTANCE = np.finfo(np.float64).max
+STUMPY_MAX_DISTANCE = np.sqrt(STUMPY_MAX_P_NORM_DISTANCE)
 STUMPY_EXCL_ZONE_DENOM = 4
