@@ -7,10 +7,11 @@ import functools
 import inspect
 
 import numpy as np
-from numba import njit, prange
+from numba import njit
 from scipy.signal import convolve
-from scipy.ndimage.filters import maximum_filter1d, minimum_filter1d
+from scipy.ndimage import maximum_filter1d, minimum_filter1d
 from scipy import linalg
+from scipy.spatial.distance import cdist
 import tempfile
 import math
 
@@ -80,17 +81,17 @@ def non_normalized(non_norm, exclude=None, replace=None):
     parameters when necessary.
 
     ```
-    def non_norm_func(Q, T, A):
+    def non_norm_func(Q, T, A_non_norm):
         ...
         return
 
 
     @non_normalized(
         non_norm_func,
-        exclude=["normalize", "A", "B"],
-        replace={"A": None},
+        exclude=["normalize", "p", "A_norm", "A_non_norm"],
+        replace={"A_norm": "A_non_norm", "other_norm": None},
     )
-    def norm_func(Q, T, B=None, normalize=True):
+    def norm_func(Q, T, A_norm=None, other_norm=None, normalize=True, p=2.0):
         ...
         return
     ```
@@ -103,13 +104,16 @@ def non_normalized(non_norm, exclude=None, replace=None):
 
     exclude : list, default None
         A list of function (or class) parameter names to exclude when comparing the
-        function (or class) signatures
+        function (or class) signatures. When `exlcude is None`, this parameter is
+        automatically set to `exclude = ["normalize", "p"]` by default.
 
     replace : dict, default None
         A dictionary of function (or class) parameter key-value pairs. Each key that
         is found as a parameter name in the `norm` function (or class) will be replaced
         by its corresponding or complementary parameter name in the `non_norm` function
-        (or class).
+        (or class) (e.g., {"norm_param": "non_norm_param"}). To remove any parameter in
+        the `norm` function (or class) that does not exist in the `non_norm` function,
+        simply set the value to `None` (i.e., {"norm_param": None}).
 
     Returns
     -------
@@ -117,7 +121,7 @@ def non_normalized(non_norm, exclude=None, replace=None):
         The desired z-normalized/non-normalized function (or class)
     """
     if exclude is None:
-        exclude = ["normalize"]
+        exclude = ["normalize", "p"]
 
     @functools.wraps(non_norm)
     def outer_wrapper(norm):
@@ -131,7 +135,7 @@ def non_normalized(non_norm, exclude=None, replace=None):
                 if replace is not None:
                     for k, v in replace.items():
                         if k in kwargs.keys():
-                            if v is None:
+                            if v is None:  # pragma: no cover
                                 _ = kwargs.pop(k)
                             else:
                                 kwargs[v] = kwargs.pop(k)
@@ -198,6 +202,13 @@ def _gpu_stimp_driver_not_found(*args, **kwargs):  # pragma: no cover
     driver_not_found()
 
 
+def _gpu_aamp_stimp_driver_not_found(*args, **kwargs):  # pragma: no cover
+    """
+    Dummy function to raise CudaSupportError driver not found error.
+    """
+    driver_not_found()
+
+
 def get_pkg_name():  # pragma: no cover
     """
     Return package name.
@@ -283,7 +294,7 @@ def check_dtype(a, dtype=np.float64):  # pragma: no cover
     a : numpy.ndarray
         NumPy array
 
-    dtype : dtype, , default np.float64
+    dtype : dtype, default np.float64
         NumPy `dtype`
 
     Raises
@@ -291,9 +302,13 @@ def check_dtype(a, dtype=np.float64):  # pragma: no cover
     TypeError
         If the array type does not match `dtype`
     """
+    if dtype == int:
+        dtype = np.int64
+    if dtype == float:
+        dtype = np.float64
     if not np.issubdtype(a.dtype, dtype):
-        msg = f"{dtype} type expected but found {a.dtype}\n"
-        msg += "Please change your input `dtype` with `.astype(np.float64)`"
+        msg = f"{dtype} dtype expected but found {a.dtype} in input array\n"
+        msg += "Please change your input `dtype` with `.astype(dtype)`"
         raise TypeError(msg)
 
     return True
@@ -445,6 +460,33 @@ def check_window_size(m, max_size=None):
         raise ValueError(f"The window size must be less than or equal to {max_size}")
 
 
+@njit(fastmath=True)
+def _sliding_dot_product(Q, T):
+    """
+    A Numba JIT-compiled implementation of the sliding window dot product.
+
+    Parameters
+    ----------
+    Q : numpy.ndarray
+        Query array or subsequence
+
+    T : numpy.ndarray
+        Time series or sequence
+
+    Returns
+    -------
+    out : numpy.ndarray
+        Sliding dot product between `Q` and `T`.
+    """
+    m = Q.shape[0]
+    k = T.shape[0] - m + 1
+    out = np.empty(k)
+    for i in range(k):
+        out[i] = np.dot(Q, T[i : i + m])
+
+    return out
+
+
 def sliding_dot_product(Q, T):
     """
     Use FFT convolution to calculate the sliding window dot product.
@@ -889,7 +931,7 @@ def _calculate_squared_distance(m, QT, μ_Q, σ_Q, M_T, Σ_T):
         if (
             σ_Q < config.STUMPY_STDDEV_THRESHOLD
             and Σ_T < config.STUMPY_STDDEV_THRESHOLD
-        ) or D_squared < config.STUMPY_D_SQUARED_THRESHOLD:
+        ) or D_squared < config.STUMPY_P_NORM_THRESHOLD:
             D_squared = 0
 
     return D_squared
@@ -897,7 +939,6 @@ def _calculate_squared_distance(m, QT, μ_Q, σ_Q, M_T, Σ_T):
 
 @njit(
     # "f8[:](i8, f8[:], f8, f8, f8[:], f8[:])",
-    parallel=True,
     fastmath=True,
 )
 def _calculate_squared_distance_profile(m, QT, μ_Q, σ_Q, M_T, Σ_T):
@@ -939,7 +980,7 @@ def _calculate_squared_distance_profile(m, QT, μ_Q, σ_Q, M_T, Σ_T):
     k = M_T.shape[0]
     D_squared = np.empty(k, dtype=np.float64)
 
-    for i in prange(k):
+    for i in range(k):
         D_squared[i] = _calculate_squared_distance(m, QT[i], μ_Q, σ_Q, M_T[i], Σ_T[i])
 
     return D_squared
@@ -947,7 +988,6 @@ def _calculate_squared_distance_profile(m, QT, μ_Q, σ_Q, M_T, Σ_T):
 
 @njit(
     # "f8[:](i8, f8[:], f8, f8, f8[:], f8[:])",
-    parallel=True,
     fastmath=True,
 )
 def calculate_distance_profile(m, QT, μ_Q, σ_Q, M_T, Σ_T):
@@ -991,43 +1031,78 @@ def calculate_distance_profile(m, QT, μ_Q, σ_Q, M_T, Σ_T):
     return np.sqrt(D_squared)
 
 
-@njit(
-    # "f8[:](f8, f8[:], f8[:])",
-    fastmath=True
-)
-def _mass_absolute(Q_squared, T_squared, QT):
+@njit(fastmath=True)
+def _p_norm_distance_profile(Q, T, p=2.0):
     """
-    A Numba JIT compiled algorithm for computing the non-normalized distance profile
-    using the MASS absolute algorithm.
+    A Numba JIT-compiled and parallelized function for computing the p-normalized
+    distance profile
 
     Parameters
     ----------
-    Q_squared : float
-        Squared query array or subsequence
+    Q : numpy.ndarray
+        Query array or subsequence
 
-    T_squared : numpy.ndarray
-        Squared time series or sequence
+    T : numpy.ndarray
+        Time series or sequence
 
-    QT : numpy.ndarray
-        Sliding window dot product of `Q` and `T`
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance.
 
     Returns
     -------
     output : numpy.ndarray
-        Unnormalized distance profile
-
-    Notes
-    -----
-    `See Mueen's Absolute Algorithm for Similarity Search \
-    <https://www.cs.unm.edu/~mueen/MASS_absolute.m>`__
+        p-normalized distance profile between `Q` and `T`
     """
-    D = Q_squared + T_squared - 2 * QT
-    D[D < config.STUMPY_D_SQUARED_THRESHOLD] = 0.0
+    m = Q.shape[0]
+    k = T.shape[0] - m + 1
+    p_norm_profile = np.empty(k, dtype=np.float64)
 
-    return np.sqrt(D)
+    if p == 2.0:
+        Q_squared = np.sum(Q * Q)
+        T_squared = np.empty(k, dtype=np.float64)
+        T_squared[0] = np.sum(T[:m] * T[:m])
+        for i in range(1, k):
+            T_squared[i] = (
+                T_squared[i - 1] - T[i - 1] * T[i - 1] + T[i + m - 1] * T[i + m - 1]
+            )
+        QT = _sliding_dot_product(Q, T)
+        for i in range(k):
+            p_norm_profile[i] = Q_squared + T_squared[i] - 2.0 * QT[i]
+    else:
+        for i in range(k):
+            p_norm_profile[i] = np.sum(np.power(np.abs(Q - T[i : i + m]), p))
+
+    return p_norm_profile
 
 
-def mass_absolute(Q, T, T_subseq_isfinite=None, T_squared=None):
+def _mass_absolute(Q, T, p=2.0):
+    """
+    A wrapper around `cdist` for computing the non-normalized distance profile
+
+    Parameters
+    ----------
+    Q : numpy.ndarray
+        Query array or subsequence
+
+    T : numpy.ndarray
+        Time series or sequence
+
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance.
+
+    Returns
+    -------
+    output : numpy.ndarray
+        Non-normalized distance profile
+    """
+    m = Q.shape[0]
+
+    return cdist(
+        rolling_window(Q, m), rolling_window(T, m), metric="minkowski", p=p
+    ).flatten()
+
+
+def mass_absolute(Q, T, T_subseq_isfinite=None, p=2.0):
     """
     Compute the non-normalized distance profile (i.e., without z-normalization) using
     the "MASS absolute" algorithm. This is a convenience wrapper around the Numba JIT
@@ -1045,8 +1120,8 @@ def mass_absolute(Q, T, T_subseq_isfinite=None, T_squared=None):
         A boolean array that indicates whether a subsequence in `T` contains a
         `np.nan`/`np.inf` value (False)
 
-    T_squared : numpy.ndarray, default None
-        Squared time series or sequence
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance.
 
     Returns
     -------
@@ -1058,10 +1133,9 @@ def mass_absolute(Q, T, T_subseq_isfinite=None, T_squared=None):
     `See Mueen's Absolute Algorithm for Similarity Search \
     <https://www.cs.unm.edu/~mueen/MASS_absolute.m>`__
     """
-    Q = Q.copy()
-    Q = np.asarray(Q)
-    check_dtype(Q)
+    Q = _preprocess(Q)
     m = Q.shape[0]
+    check_window_size(m, max_size=Q.shape[-1])
 
     if Q.ndim == 2 and Q.shape[1] == 1:  # pragma: no cover
         Q = Q.flatten()
@@ -1069,9 +1143,7 @@ def mass_absolute(Q, T, T_subseq_isfinite=None, T_squared=None):
     if Q.ndim != 1:  # pragma: no cover
         raise ValueError(f"Q is {Q.ndim}-dimensional and must be 1-dimensional. ")
 
-    T = T.copy()
-    T = np.asarray(T)
-    check_dtype(T)
+    T = _preprocess(T)
     n = T.shape[0]
 
     if T.ndim == 2 and T.shape[1] == 1:  # pragma: no cover
@@ -1092,17 +1164,13 @@ def mass_absolute(Q, T, T_subseq_isfinite=None, T_squared=None):
     else:
         if T_subseq_isfinite is None:
             T, T_subseq_isfinite = preprocess_non_normalized(T, m)
-        QT = sliding_dot_product(Q, T)
-        Q_squared = np.sum(Q * Q)
-        if T_squared is None:
-            T_squared = np.sum(rolling_window(T * T, m), axis=-1)
-        distance_profile[:] = _mass_absolute(Q_squared, T_squared, QT)
+        distance_profile[:] = _mass_absolute(Q, T, p)
         distance_profile[~T_subseq_isfinite] = np.inf
 
     return distance_profile
 
 
-def _mass_absolute_distance_matrix(Q, T, m, distance_matrix):
+def _mass_absolute_distance_matrix(Q, T, m, distance_matrix, p=2.0):
     """
     Compute the full non-normalized (i.e., without z-normalization) distance matrix
     between all of the subsequences of `Q` and `T` using the MASS absolute algorithm
@@ -1121,14 +1189,20 @@ def _mass_absolute_distance_matrix(Q, T, m, distance_matrix):
     distance_matrix : numpy.ndarray
         The full output distance matrix. This is mandatory since it may be reused.
 
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance.
+
     Returns
     -------
     None
     """
-    k, l = distance_matrix.shape
-
-    for i in range(k):
-        distance_matrix[i, :] = mass_absolute(Q[i : i + m], T)
+    cdist(
+        rolling_window(Q, m),
+        rolling_window(T, m),
+        out=distance_matrix,
+        metric="minkowski",
+        p=p,
+    )
 
 
 def mueen_calculate_distance_profile(Q, T):
@@ -1208,6 +1282,12 @@ def _mass(Q, T, QT, μ_Q, σ_Q, M_T, Σ_T):
     A Numba JIT compiled algorithm for computing the distance profile using the MASS
     algorithm.
 
+    This private function assumes only finite numbers in your inputs and it is the
+    responsibility of the user to pre-process and post-process their results if the
+    original time series contains `np.nan`/`np.inf` values. Failure to do so will
+    result in incorrect outputs. See `core.mass` for common pre-processing and
+    post-processing procedures.
+
     Parameters
     ----------
     Q : numpy.ndarray
@@ -1255,10 +1335,10 @@ def _mass(Q, T, QT, μ_Q, σ_Q, M_T, Σ_T):
 
 @non_normalized(
     mass_absolute,
-    exclude=["normalize", "M_T", "Σ_T", "T_subseq_isfinite", "T_squared"],
-    replace={"M_T": None, "Σ_T": None},
+    exclude=["normalize", "M_T", "Σ_T", "T_subseq_isfinite", "p"],
+    replace={"M_T": "T_subseq_isfinite", "Σ_T": None},
 )
-def mass(Q, T, M_T=None, Σ_T=None, normalize=True):
+def mass(Q, T, M_T=None, Σ_T=None, normalize=True, p=2.0):
     """
     Compute the distance profile using the MASS algorithm
 
@@ -1283,6 +1363,10 @@ def mass(Q, T, M_T=None, Σ_T=None, normalize=True):
         Otherwise, this function gets re-routed to its complementary non-normalized
         equivalent set in the `@core.non_normalized` function decorator.
 
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance. This parameter is
+        ignored when `normalize == True`.
+
     Returns
     -------
     distance_profile : numpy.ndarray
@@ -1312,10 +1396,9 @@ def mass(Q, T, M_T=None, Σ_T=None, normalize=True):
     ...     np.array([584., -11., 23., 79., 1001., 0., -19.]))
     array([3.18792463e+00, 1.11297393e-03, 3.23874018e+00, 3.34470195e+00])
     """
-    Q = Q.copy()
-    Q = np.asarray(Q)
-    check_dtype(Q)
+    Q = _preprocess(Q)
     m = Q.shape[0]
+    check_window_size(m, max_size=Q.shape[-1])
 
     if Q.ndim == 2 and Q.shape[1] == 1:  # pragma: no cover
         Q = Q.flatten()
@@ -1323,9 +1406,7 @@ def mass(Q, T, M_T=None, Σ_T=None, normalize=True):
     if Q.ndim != 1:  # pragma: no cover
         raise ValueError(f"Q is {Q.ndim}-dimensional and must be 1-dimensional. ")
 
-    T = T.copy()
-    T = np.asarray(T)
-    check_dtype(T)
+    T = _preprocess(T)
     n = T.shape[0]
 
     if T.ndim == 2 and T.shape[1] == 1:  # pragma: no cover
@@ -1425,17 +1506,17 @@ def _get_QT(start, T_A, T_B, m):
     # ["(f8[:], i8, i8)", "(f8[:, :], i8, i8)"],
     fastmath=True
 )
-def apply_exclusion_zone(D, idx, excl_zone):
+def _apply_exclusion_zone(a, idx, excl_zone, val):
     """
     Apply an exclusion zone to an array (inplace), i.e. set all values
-    to np.inf in a window around a given index.
+    to `val` in a window around a given index.
 
-    All values in D in [idx - excl_zone, idx + excl_zone] (endpoints included)
-    will be set to np.inf.
+    All values in a in [idx - excl_zone, idx + excl_zone] (endpoints included)
+    will be set to `val`.
 
     Parameters
     ----------
-    D : numpy.ndarray
+    a : numpy.ndarray
         The array you want to apply the exclusion zone to
 
     idx : int
@@ -1443,10 +1524,63 @@ def apply_exclusion_zone(D, idx, excl_zone):
 
     excl_zone : int
         Size of the exclusion zone.
+
+    val : float or bool
+        The elements within the exclusion zone will be set to this value
     """
     zone_start = max(0, idx - excl_zone)
-    zone_stop = min(D.shape[-1], idx + excl_zone)
-    D[..., zone_start : zone_stop + 1] = np.inf
+    zone_stop = min(a.shape[-1], idx + excl_zone)
+    a[..., zone_start : zone_stop + 1] = val
+
+
+def apply_exclusion_zone(a, idx, excl_zone, val):
+    """
+    Apply an exclusion zone to an array (inplace), i.e. set all values
+    to `val` in a window around a given index.
+
+    All values in a in [idx - excl_zone, idx + excl_zone] (endpoints included)
+    will be set to `val`. This is a convenience wrapper around the Numba JIT-compiled
+    `_apply_exclusion_zone` function.
+
+    Parameters
+    ----------
+    a : numpy.ndarray
+        The array you want to apply the exclusion zone to
+
+    idx : int
+        The index around which the window should be centered
+
+    excl_zone : int
+        Size of the exclusion zone.
+
+    val : float or bool
+        The elements within the exclusion zone will be set to this value
+    """
+    check_dtype(a, dtype=type(val))
+    _apply_exclusion_zone(a, idx, excl_zone, val)
+
+
+def _preprocess(T):
+    """
+    Creates a copy of the time series, transposes all dataframes, converts to
+    `numpy.ndarray`, and checks the `dtype`
+
+    Parameters
+    ----------
+    T : numpy.ndarray
+        Time series or sequence
+
+    Returns
+    -------
+    T : numpy.ndarray
+        Modified time series
+    """
+    T = T.copy()
+    T = transpose_dataframe(T)
+    T = np.asarray(T)
+    check_dtype(T)
+
+    return T
 
 
 def preprocess(T, m):
@@ -1475,12 +1609,8 @@ def preprocess(T, m):
     Σ_T : numpy.ndarray
         Rolling standard deviation
     """
-    T = T.copy()
-    T = transpose_dataframe(T)
-    T = np.asarray(T)
-    check_dtype(T)
+    T = _preprocess(T)
     check_window_size(m, max_size=T.shape[-1])
-
     T[np.isinf(T)] = np.nan
     M_T, Σ_T = compute_mean_std(T, m)
     T[np.isnan(T)] = 0
@@ -1515,12 +1645,8 @@ def preprocess_non_normalized(T, m):
         A boolean array that indicates whether a subsequence in `T` contains a
         `np.nan`/`np.inf` value (False)
     """
-    T = T.copy()
-    T = transpose_dataframe(T)
-    T = np.asarray(T)
-    check_dtype(T)
+    T = _preprocess(T)
     check_window_size(m, max_size=T.shape[-1])
-
     T_subseq_isfinite = rolling_isfinite(T, m)
     T[~np.isfinite(T)] = 0.0
 
@@ -1631,7 +1757,6 @@ def array_to_temp_file(a):
 
 @njit(
     # "i8[:](i8[:], i8, i8, i8)",
-    parallel=True,
     fastmath=True,
 )
 def _count_diagonal_ndist(diags, m, n_A, n_B):
@@ -1659,7 +1784,7 @@ def _count_diagonal_ndist(diags, m, n_A, n_B):
         Counts of distances computed along each diagonal of interest
     """
     diag_ndist_counts = np.zeros(diags.shape[0], dtype=np.int64)
-    for diag_idx in prange(diags.shape[0]):
+    for diag_idx in range(diags.shape[0]):
         k = diags[diag_idx]
         if k >= 0:
             diag_ndist_counts[diag_idx] = min(n_B - m + 1 - k, n_A - m + 1)
@@ -1717,6 +1842,58 @@ def _get_array_ranges(a, n_chunks, truncate):
     return array_ranges
 
 
+@njit(
+    # "i8[:, :](i8, i8, b1)"
+)
+def _get_ranges(size, n_chunks, truncate):
+    """
+    Given a single input integer value, split an array of that length `size` evenly into
+    `n_chunks`.
+
+    This function is different from `_get_array_ranges` in that it does not take into
+    account the contents of the array and, instead, assumes that we are chunking up
+    `np.ones(size, dtype=np.int64)`. Additionally, the non-truncated sections may not
+    all appear at the end of the returned away (i.e., they may be scattered throughout
+    different rows of the array) but may be identified as having the same start and
+    stop indices.
+
+    Parameters
+    ----------
+    size : int
+        The size or length of an array to chunk
+
+    n_chunks : int
+        Number of chunks to split the array into
+
+    truncate : bool
+        If `truncate=True`, truncate the rows of `array_ranges` if there are not enough
+        elements in `a` to be chunked up into `n_chunks`.  Otherwise, if
+        `truncate=False`, all extra chunks will have their start and stop indices set
+        to `a.shape[0]`.
+
+    Returns
+    -------
+    array_ranges : numpy.ndarray
+        A two column array where each row consists of a start and (exclusive) stop index
+        pair. The first column contains the start indices and the second column
+        contains the stop indices.
+    """
+    a = np.ones(size, dtype=np.int64)
+    array_ranges = np.zeros((n_chunks, 2), dtype=np.int64)
+    if a.shape[0] > 0 and n_chunks > 0:
+        cumsum = a.cumsum()
+        insert = np.linspace(0, a.sum(), n_chunks + 1)[1:-1]
+        idx = 1 + np.searchsorted(cumsum, insert)
+        array_ranges[1:, 0] = idx  # Fill the first column with start indices
+        array_ranges[:-1, 1] = idx  # Fill the second column with exclusive stop indices
+        array_ranges[-1, 1] = a.shape[0]  # Handle the stop index for the final chunk
+
+    if truncate:
+        array_ranges = array_ranges[array_ranges[:, 0] != array_ranges[:, 1]]
+
+    return array_ranges
+
+
 def _rolling_isfinite_1d(a, w):
     """
     Determine if all elements in each rolling window `isfinite`
@@ -1924,3 +2101,296 @@ def _idx_to_mp(I, T, m, normalize=True):
     P[I < 0] = np.inf
 
     return P
+
+
+@njit(fastmath=True)
+def _total_diagonal_ndists(tile_lower_diag, tile_upper_diag, tile_height, tile_width):
+    """
+    Count the total number of distances covered by a range of diagonals
+
+    Parameters
+    ----------
+    tile_lower_diag : int
+        The (inclusive) lower diagonal index
+
+    tile_upper_diag : int
+        The (exclusive) upper diagonal index
+
+    tile_height : int
+        The height of the tile
+
+    tile_width : int
+        The width of the tile
+
+    Returns
+    -------
+    out : int
+        The total number of distances
+
+    Notes
+    -----
+    This function essentially uses the "shoelace formula" to determine the area
+    of a simple polygon whose vertices are described by their Cartesian coordinates
+    in a plane.
+    """
+    if tile_width < tile_height:
+        # Transpose inputs, adjust for inclusive/exclusive diags
+        tile_width, tile_height = tile_height, tile_width
+        tile_lower_diag, tile_upper_diag = 1 - tile_upper_diag, 1 - tile_lower_diag
+
+    if tile_lower_diag > tile_upper_diag:  # pragma: no cover
+        # Swap diags
+        tile_lower_diag, tile_upper_diag = tile_upper_diag, tile_lower_diag
+
+    min_tile_diag = 1 - tile_height
+    max_tile_diag = tile_width  # Exclusive
+
+    if (
+        tile_lower_diag < min_tile_diag
+        or tile_upper_diag < min_tile_diag
+        or tile_lower_diag > max_tile_diag
+        or tile_upper_diag > max_tile_diag
+    ):
+
+        return 0
+
+    if tile_lower_diag == min_tile_diag and tile_upper_diag == max_tile_diag:
+        return tile_height * tile_width
+
+    # Determine polygon shape and establish vertices
+    if tile_lower_diag <= 0 and tile_upper_diag <= 0:
+        # lower trapezoid/triangle
+        lower_ndists = tile_height + tile_lower_diag
+        upper_ndists = tile_height + tile_upper_diag
+        vertices = np.array(
+            [
+                [tile_lower_diag, 0],
+                [tile_upper_diag, 0],
+                [-tile_height, upper_ndists],
+                [-tile_height, lower_ndists],
+            ]
+        )
+    elif tile_lower_diag <= 0 and 0 < tile_upper_diag <= tile_width - tile_height:
+        # irregular hexagon/diamond
+        lower_ndists = tile_height + tile_lower_diag
+        upper_ndists = min(tile_height, tile_width - tile_upper_diag)
+        vertices = np.array(
+            [
+                [tile_lower_diag, 0],
+                [0, 0],
+                [0, tile_upper_diag],
+                [-upper_ndists, tile_upper_diag + upper_ndists],
+                [-tile_height, lower_ndists],
+            ]
+        )
+    elif tile_lower_diag <= 0 and tile_upper_diag >= tile_width - tile_height:
+        # irregular hexagon/diamond
+        lower_ndists = tile_height + tile_lower_diag
+        upper_ndists = min(tile_height, tile_width - tile_upper_diag)
+        vertices = np.array(
+            [
+                [tile_lower_diag, 0],
+                [0, 0],
+                [0, tile_upper_diag],
+                [-upper_ndists, tile_upper_diag + upper_ndists],
+                [-tile_height, tile_width],
+                [-tile_height, lower_ndists],
+            ]
+        )
+    elif (
+        0 < tile_lower_diag <= tile_width - tile_height
+        and 0 < tile_upper_diag <= tile_width - tile_height
+    ):
+        # parallelogram
+        lower_ndists = min(tile_height, tile_width - tile_lower_diag)
+        upper_ndists = min(tile_height, tile_width - tile_upper_diag)
+        vertices = np.array(
+            [
+                [0, tile_lower_diag],
+                [0, tile_upper_diag],
+                [-upper_ndists, tile_upper_diag + upper_ndists],
+                [-tile_height, tile_lower_diag + lower_ndists],
+            ]
+        )
+    elif (
+        0 < tile_lower_diag <= tile_width - tile_height
+        and tile_upper_diag > tile_width - tile_height
+    ):
+        # upper diamond
+        lower_ndists = min(tile_height, tile_width - tile_lower_diag)
+        upper_ndists = tile_width - tile_upper_diag
+        vertices = np.array(
+            [
+                [0, tile_lower_diag],
+                [0, tile_upper_diag],
+                [-upper_ndists, tile_upper_diag + upper_ndists],
+                [-tile_height, tile_width],
+                [-tile_height, tile_lower_diag + lower_ndists],
+            ]
+        )
+    else:
+        # tile_lower_diag > tile_width - tile_height and
+        # tile_upper_diag > tile_width - tile_height
+        # upper trapezoid
+        lower_ndists = tile_width - tile_lower_diag
+        upper_ndists = tile_width - tile_upper_diag
+        vertices = np.array(
+            [
+                [0, tile_lower_diag],
+                [0, tile_upper_diag],
+                [-upper_ndists, tile_upper_diag + upper_ndists],
+                [-lower_ndists, tile_width],
+            ]
+        )
+
+    # Shoelace formula
+    i = np.arange(vertices.shape[0])
+    total_diagonal_ndists = np.abs(
+        np.sum(
+            vertices[i, 0] * vertices[i - 1, 1] - vertices[i, 1] * vertices[i - 1, 0]
+        )
+        / 2
+    )
+    # Account for over/under-counting due to jagged upper/lower diagonal shapes
+    total_diagonal_ndists += lower_ndists / 2 - upper_ndists / 2
+
+    return int(total_diagonal_ndists)
+
+
+def _bfs_indices(n):
+    """
+    Generate the level order indices from the implicit construction of a binary
+    search tree followed by a breadth first (level order) search.
+
+    Example:
+
+    If `n = 10` then the corresponding (zero-based index) balanced binary tree is:
+
+                5
+               * *
+              *   *
+             *     *
+            *       *
+           *         *
+          2           8
+         * *         * *
+        *   *       *   *
+       *     *     *     *
+      1       4   7       9
+     * *     *
+    0   3   6
+
+    And if we traverse the nodes at each level from left to right then the breadth
+    first search indices would be `[5, 2, 8, 1, 4, 7, 9, 0, 3, 6]`. In this function,
+    we avoid/skip the explicit construction of the binary tree and directly output
+    the desired indices efficiently.
+
+    Parameters
+    ----------
+    n : int
+        The number indices to generate the ordered indices for
+
+    Returns
+    -------
+    level_idx : numpy.ndarray
+        The breadth first search (level order) indices
+    """
+    if n == 1:  # pragma: no cover
+        return np.array([0], dtype=np.int64)
+
+    nlevel = np.floor(np.log2(n) + 1).astype(np.int64)
+    nindices = np.power(2, np.arange(nlevel))
+    cumsum_nindices = np.cumsum(nindices)
+    nindices[-1] = n - cumsum_nindices[np.searchsorted(cumsum_nindices, n) - 1]
+
+    indices = np.empty((2, nindices.max()), dtype=np.int64)
+    indices[0, 0] = 0
+    indices[1, 0] = n
+    tmp_indices = np.empty((2, 2 * nindices.max()), dtype=np.int64)
+
+    out = np.empty(n, dtype=np.int64)
+    out_idx = 0
+
+    for nidx in nindices:
+        level_indices = (indices[0, :nidx] + indices[1, :nidx]) // 2
+
+        if out_idx + len(level_indices) < n:
+            tmp_indices[0, 0 : 2 * nidx : 2] = indices[0, :nidx]
+            tmp_indices[0, 1 : 2 * nidx : 2] = level_indices + 1
+            tmp_indices[1, 0 : 2 * nidx : 2] = level_indices
+            tmp_indices[1, 1 : 2 * nidx : 2] = indices[1, :nidx]
+
+            mask = tmp_indices[0, : 2 * nidx] < tmp_indices[1, : 2 * nidx]
+            mask_sum = np.count_nonzero(mask)
+            indices[0, :mask_sum] = tmp_indices[0, : 2 * nidx][mask]
+            indices[1, :mask_sum] = tmp_indices[1, : 2 * nidx][mask]
+
+        # for level_idx in level_indices:
+        #     yield level_idx
+
+        out[out_idx : out_idx + len(level_indices)] = level_indices
+        out_idx += len(level_indices)
+
+    return out
+
+
+def _contrast_pan(pan, threshold, bfs_indices, n_processed):
+    """
+    Center the pan matrix profile (inplace) around the desired distance threshold
+    in order to increase the contrast
+
+    Parameters
+    ----------
+    pan : numpy.ndarray
+        The pan matrix profile
+
+    threshold : float
+        The distance threshold value in which to center the pan matrix profile around
+
+    bfs_indices : numpy.ndarray
+        The breadth-first-search indices
+
+    n_processed : numpy.ndarray
+        The number of breadth-first-search indices to apply contrast to
+
+    Returns
+    -------
+    None
+    """
+    idx = bfs_indices[:n_processed]
+    l = n_processed * pan.shape[1]
+    tmp = pan[idx].argsort(kind="mergesort", axis=None)
+    ranks = np.empty(l, dtype=np.int64)
+    ranks[tmp] = np.arange(l).astype(np.int64)
+
+    percentile = np.full(ranks.shape, np.nan)
+    percentile[:l] = np.linspace(0, 1, l)
+    percentile = percentile[ranks].reshape(pan[idx].shape)
+    pan[idx] = 1.0 / (1.0 + np.exp(-10 * (percentile - threshold)))
+
+
+def _binarize_pan(pan, threshold, bfs_indices, n_processed):
+    """
+    Binarize the pan matrix profile (inplace) such all values below the `threshold`
+    are set to `0.0` and all values above the `threshold` are set to `1.0`.
+
+    Parameters
+    ----------
+    pan : numpy.ndarray
+        The pan matrix profile
+
+    threshold : float
+        The distance threshold value in which to center the pan matrix profile around
+
+    bfs_indices : numpy.ndarray
+        The breadth-first-search indices
+
+    n_processed : numpy.ndarray
+        The number of breadth-first-search indices to binarize
+
+    Returns
+    -------
+    None
+    """
+    idx = bfs_indices[:n_processed]
+    pan[idx] = np.where(pan[idx] <= threshold, 0.0, 1.0)
