@@ -11,7 +11,9 @@ from .gpu_aampdist import gpu_aampdist
 
 
 @core.non_normalized(gpu_aampdist)
-def gpu_mpdist(T_A, T_B, m, percentage=0.05, k=None, device_id=0, normalize=True):
+def gpu_mpdist(
+    T_A, T_B, m, percentage=0.05, k=None, device_id=0, normalize=True, p=2.0
+):
     """
     Compute the z-normalized matrix profile distance (MPdist) measure between any two
     time series with one or more GPU devices
@@ -53,6 +55,10 @@ def gpu_mpdist(T_A, T_B, m, percentage=0.05, k=None, device_id=0, normalize=True
         Otherwise, this function gets re-routed to its complementary non-normalized
         equivalent set in the `@core.non_normalized` function decorator.
 
+    p : float, default 2.0
+        The p-norm to apply for computing the Minkowski distance. This parameter is
+        ignored when `normalize == True`.
+
     Returns
     -------
     MPdist : float
